/***************************************************************************
 *   Copyright (C) 2004 by Ingo Stierand                                   *
 *   ing.stierand@web.de                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "kinstaller.h"
#include "pref.h"

#include <qdragobject.h>
#include <kprinter.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>

#include <kglobal.h>
#include <klocale.h>
#include <kiconloader.h>
#include <kdeversion.h>
#include <kmenubar.h>
#include <kmessagebox.h>
#include <kstatusbar.h>
#include <kkeydialog.h>
#include <kaccel.h>
#include <kio/netaccess.h>
#include <kfiledialog.h>
#include <kdirselectdialog.h>
#include <kconfig.h>
#include <kurl.h>
#include <kurldrag.h>
#include <kurlrequesterdlg.h>

#include <kedittoolbar.h>

#include <kstdaccel.h>
#include <kaction.h>
#include <kstdaction.h>


KInstaller::KInstaller()
    : KMainWindow( 0, "KInstaller" ),
      m_view(new KInstallerView(this)),
      m_printer(0)
{
    // accept dnd
    setAcceptDrops(false);

    // tell the KMainWindow that this is indeed the main widget
    setCentralWidget(m_view);

    // then, setup our actions
    setupActions();

    // and a status bar
    statusBar()->show();

    // apply the saved mainwindow settings, if any, and ask the mainwindow
    // to automatically save settings if changed: window size, toolbar
    // position, icon size, etc.
    setAutoSaveSettings();

    // allow the view to change the statusbar and caption
/*
    connect(m_view, SIGNAL(signalChangeStatusbar(const QString&)),
            this,   SLOT(changeStatusbar(const QString&)));
    connect(m_view, SIGNAL(signalChangeCaption(const QString&)),
            this,   SLOT(changeCaption(const QString&)));
*/
    connect(m_view, SIGNAL(signalSelectSourceClicked()),
            this,   SLOT(slotSelectSourceClicked()));
    connect(m_view, SIGNAL(signalSelectTempDirClicked()),
            this,   SLOT(slotSelectTempDirClicked()));
    connect(m_view, SIGNAL(signalSelectPkgpDirClicked()),
            this,   SLOT(slotSelectPkgDirClicked()));
    connect(m_view, SIGNAL(signalGuessKDEBaseClicked()),
            this,   SLOT(slotGuessKDEBaseClicked()));
    connect(m_view, SIGNAL(signalInstallClicked()),
            this,   SLOT(slotInstallClicked()));
    
    vRunning = false;
    vCanceled = false;
    
    vProcess = new KInstallerProcess(this);
    connect(vProcess,SIGNAL(signalProcessStdout(const QString& )),
            this,    SLOT(slotProcessStdout(const QString& )));
    connect(vProcess,SIGNAL(signalProcessStderr(const QString& )),
            this,    SLOT(slotProcessStderr(const QString& )));

    readGlobalConfig();
}


KInstaller::~KInstaller()
{
}


void KInstaller::load(const KURL& url)
{
    QString target;
    // the below code is what you should normally do.  in this
    // example case, we want the url to our own.  you probably
    // want to use this code instead for your app

    #if 0
    // download the contents
    if (KIO::NetAccess::download(url, target))
    {
        // set our caption
        setCaption(url);

        // load in the file (target is always local)
        loadFile(target);

        // and remove the temp file
        KIO::NetAccess::removeTempFile(target);
    }
    #endif

    setCaption(url.prettyURL());
//    m_view->openURL(url);
}


void KInstaller::setupActions()
{
//    KStdAction::openNew(this, SLOT(fileNew()), actionCollection());
    KStdAction::open(this, SLOT(fileOpen()), actionCollection());
//    KStdAction::save(this, SLOT(fileSave()), actionCollection());
//    KStdAction::saveAs(this, SLOT(fileSaveAs()), actionCollection());
//    KStdAction::print(this, SLOT(filePrint()), actionCollection());
    KStdAction::quit(kapp, SLOT(quit()), actionCollection());

//    m_toolbarAction = KStdAction::showToolbar(this, SLOT(optionsShowToolbar()), actionCollection());
//    m_statusbarAction = KStdAction::showStatusbar(this, SLOT(optionsShowStatusbar()), actionCollection());

//    KStdAction::keyBindings(this, SLOT(optionsConfigureKeys()), actionCollection());
//    KStdAction::configureToolbars(this, SLOT(optionsConfigureToolbars()), actionCollection());
//    KStdAction::preferences(this, SLOT(optionsPreferences()), actionCollection());

    // this doesn't do anything useful.  it's just here to illustrate
    // how to insert a custom menu and menu item
/*
        KAction *custom = new KAction(i18n("Cus&tom Menuitem"), 0,
                                  this, SLOT(optionsPreferences()),
                                  actionCollection(), "custom_action");
    */
    
    createGUI();
    toolBar()->hide();
}


void KInstaller::saveProperties(KConfig *config)
{
    // the 'config' object points to the session managed
    // config file.  anything you write here will be available
    // later when this app is restored

/*
      if (!m_view->currentURL().isEmpty()) {
#if KDE_IS_VERSION(3,1,3)
        config->writePathEntry("lastURL", m_view->currentURL());
#else
        config->writeEntry("lastURL", m_view->currentURL());
#endif
    }
  */
}


void KInstaller::readProperties(KConfig *config)
{
    // the 'config' object points to the session managed
    // config file.  this function is automatically called whenever
    // the app is being restored.  read in here whatever you wrote
    // in 'saveProperties'

//    QString url = config->readPathEntry("lastURL");

//    if (!url.isEmpty())
//        m_view->openURL(KURL(url));
}


#define CONFIG_GROUPGENERAL     "General"
#define CONFIG_KEYSOURCEURL     "sourceUrl"
#define CONFIG_KEYKDEBASE       "kdeBase"
#define CONFIG_KEYTEMPDIR       "tempDir"
#define CONFIG_KEYPKGDIR        "pkgDir"
#define CONFIG_KEYINSTALLTYPE   "installType"
#define CONFIG_KEYUSECONFIGOPTIONS "useConfigureOptions"
#define CONFIG_KEYCONFIGOPTIONS "configureOptions"


/*!
 */
void KInstaller::readGlobalConfig()
{
  KConfig * config = KGlobal::config();
  
  
  // read general options
  config->setGroup(CONFIG_GROUPGENERAL);
  m_view->setSourceURL(config->readEntry(CONFIG_KEYSOURCEURL,""));
  m_view->setKDEBase(config->readEntry(CONFIG_KEYKDEBASE,"/opt/kde3"));
  m_view->setTempDir(config->readEntry(CONFIG_KEYTEMPDIR,"/tmp"));
  m_view->setPkgDir(config->readEntry(CONFIG_KEYPKGDIR,""));
  if (config->readNumEntry(CONFIG_KEYINSTALLTYPE,0) == 0)
    m_view->setInstallType(KInstallerView::Temporary);
  else
    m_view->setInstallType(KInstallerView::Persistent);
  m_view->setUseConfigureOptions(config->readBoolEntry(CONFIG_KEYUSECONFIGOPTIONS,false));
  m_view->setConfigureOptions(config->readEntry(CONFIG_KEYCONFIGOPTIONS,""));
}


/*!
 */
void KInstaller::writeGlobalConfig()
{
  KConfig * config = KGlobal::config();
    
  
  // write general options
  config->setGroup(CONFIG_GROUPGENERAL);
  config->writeEntry(CONFIG_KEYSOURCEURL,m_view->sourceURL());
  config->writeEntry(CONFIG_KEYKDEBASE,m_view->KDEBase());
  config->writeEntry(CONFIG_KEYTEMPDIR,m_view->tempDir());
  config->writeEntry(CONFIG_KEYPKGDIR,m_view->pkgDir());
  if (m_view->installType() == KInstallerView::Temporary)
    config->writeEntry(CONFIG_KEYINSTALLTYPE,0);
  else
    config->writeEntry(CONFIG_KEYINSTALLTYPE,1);
  config->writeEntry(CONFIG_KEYCONFIGOPTIONS,m_view->configureOptions());
  config->writeEntry(CONFIG_KEYUSECONFIGOPTIONS,m_view->useConfigureOptions());
      
  // We need to call 'sync' to write back all changes
  //  Maybe this is a bug in KDE? The config is only written
  //  when internal keys are changed
  config->sync();
}


void KInstaller::dragEnterEvent(QDragEnterEvent *event)
{
    // accept uri drops only
    event->accept(KURLDrag::canDecode(event));
}


void KInstaller::dropEvent(QDropEvent *event)
{
    // this is a very simplistic implementation of a drop event.  we
    // will only accept a dropped URL.  the Qt dnd code can do *much*
    // much more, so please read the docs there
    KURL::List urls;

    // see if we can decode a URI.. if not, just ignore it
    if (KURLDrag::decode(event, urls) && !urls.isEmpty())
    {
        // okay, we have a URI.. process it
        const KURL &url = urls.first();

        // load in the file
        load(url);
    }
}


void KInstaller::fileNew()
{
    // this slot is called whenever the File->New menu is selected,
    // the New shortcut is pressed (usually CTRL+N) or the New toolbar
    // button is clicked

    // create a new window
    (new KInstaller)->show();
}


void KInstaller::fileOpen()
{
    // this slot is called whenever the File->Open menu is selected,
    // the Open shortcut is pressed (usually CTRL+O) or the Open toolbar
    // button is clicked
/*
    // this brings up the generic open dialog
    KURL url = KURLRequesterDlg::getURL(QString::null, this, i18n("Open Location") );
*/
  // standard filedialog
  QString dir;
  
  if (m_view->sourceURL().isEmpty())
    dir = QDir::current().absPath();
  else
    dir = m_view->sourceURL();
  
  KURL url = KFileDialog::getOpenURL(dir,
                                     "*.tar *.tar.gz *.tgz *.tar.bz2|tar ball (.tar .tar.gz .tgz .tar.bz2)\n*|All files",
                                     this, i18n("Select source package"));
  if (!url.isEmpty())
  {
    m_view->setSourceURL(url.path());
  }
}


void KInstaller::filePrint()
{
    // this slot is called whenever the File->Print menu is selected,
    // the Print shortcut is pressed (usually CTRL+P) or the Print toolbar
    // button is clicked
/*
    if (!m_printer) m_printer = new KPrinter;
    if (m_printer->setup(this))
    {
        // setup the printer.  with Qt, you always "print" to a
        // QPainter.. whether the output medium is a pixmap, a screen,
        // or paper
        QPainter p;
        p.begin(m_printer);

        // we let our view do the actual printing
        QPaintDeviceMetrics metrics(m_printer);
        m_view->print(&p, metrics.height(), metrics.width());

        // and send the result to the printer
        p.end();
    }
*/
}


void KInstaller::optionsShowToolbar()
{
    // this is all very cut and paste code for showing/hiding the
    // toolbar
    if (m_toolbarAction->isChecked())
        toolBar()->show();
    else
        toolBar()->hide();
}


void KInstaller::optionsShowStatusbar()
{
    // this is all very cut and paste code for showing/hiding the
    // statusbar
    if (m_statusbarAction->isChecked())
        statusBar()->show();
    else
        statusBar()->hide();
}


/*
void KInstaller::optionsConfigureKeys()
{
    KKeyDialog::configureKeys(actionCollection(), "kinstallerui.rc");
}
*/


void KInstaller::optionsConfigureToolbars()
{
    // use the standard toolbar editor
#if defined(KDE_MAKE_VERSION)
# if KDE_VERSION >= KDE_MAKE_VERSION(3,1,0)
    saveMainWindowSettings(KGlobal::config(), autoSaveGroup());
# else
    saveMainWindowSettings(KGlobal::config());
# endif
#else
    saveMainWindowSettings(KGlobal::config());
#endif
}


void KInstaller::newToolbarConfig()
{
    // this slot is called when user clicks "Ok" or "Apply" in the toolbar editor.
    // recreate our GUI, and re-apply the settings (e.g. "text under icons", etc.)
    createGUI();

#if defined(KDE_MAKE_VERSION)
# if KDE_VERSION >= KDE_MAKE_VERSION(3,1,0)
    applyMainWindowSettings(KGlobal::config(), autoSaveGroup());
# else
    applyMainWindowSettings(KGlobal::config());
# endif
#else
    applyMainWindowSettings(KGlobal::config());
#endif
}


void KInstaller::optionsPreferences()
{
    // popup some sort of preference dialog, here
    KInstallerPreferences dlg;
    if (dlg.exec())
    {
        // redo your settings
    }
}


void KInstaller::changeStatusbar(const QString& text)
{
    // display the text on the statusbar
    statusBar()->message(text);
}


void KInstaller::changeCaption(const QString& text)
{
    // display the text on the caption
    setCaption(text);
}


/*!
    \fn KInstaller::queryExit()
 */
bool KInstaller::queryExit()
{
  writeGlobalConfig();
  
  return(!vProcess->isRunning());
}

/*!
    \fn KInstaller::slotSelectSourceClicked()
 */
void KInstaller::slotSelectSourceClicked()
{
  fileOpen();
}


/*!
    \fn KInstaller::slotSelectTempDirClicked()
 */
void KInstaller::slotSelectTempDirClicked()
{
  QString dir;
  
  
  if (m_view->tempDir().isEmpty())
    dir = QDir::current().absPath();
  else
    dir = m_view->tempDir();
  
  dir = selectDirectory(dir,i18n("select temporary directory"));
  if (!dir.isEmpty())
  {
    m_view->setTempDir(dir);
  }
}


/*!
    \fn KInstaller::slotSelectPkgDirClicked()
 */
void KInstaller::slotSelectPkgDirClicked()
{
  QString dir;
  
  
  if (m_view->pkgDir().isEmpty())
    dir = QDir::current().absPath();
  else
    dir = m_view->pkgDir();
  
  dir = selectDirectory(dir,i18n("select package directory"));
  if (!dir.isEmpty())
  {
    m_view->setPkgDir(dir);
  }
}


/*!
    \fn KInstaller::slotGuessKDEBaseClicked()
 */
void KInstaller::slotGuessKDEBaseClicked()
{
  QStringList vBaseDirs;
  
  
  vBaseDirs.append("/opt/kde3");
  vBaseDirs.append("/usr");
  vBaseDirs.append("/usr/local");
  
  for (QStringList::Iterator it = vBaseDirs.begin(); it != vBaseDirs.end(); ++it)
  {
    if (testKDEBaseDir(*it))
    {
      m_view->setKDEBase(*it);
      return;
    }
  }
  
  showInfo("Sorry but KInstaller wasn't able to determine the KDE base"\
            "directory. Please enter the directory by hand.");
}


/*!
    \fn KInstaller::selectDirectory(const QString & startDir, const QString & caption)
 */
QString KInstaller::selectDirectory(const QString & startDir, const QString & caption)
{
  QString dir;
  
  
  dir = KFileDialog::getExistingDirectory(startDir,this,caption);
  return(dir);
}


#define processStart m_view->toggleInstallButton(true,true);\
                     vRunning = true; vCanceled = false;\
                     menuBar()->setEnabled(false);
                     
#define processReady m_view->toggleInstallButton(false,true);\
                     vRunning = false; vCanceled = false;\
                     menuBar()->setEnabled(true);
                     
#define processCancelRequest if (vCanceled) {\
                        processCancel(pckdir);\
                        menuBar()->setEnabled(true);\
                        return;}

/*!
    \fn KInstaller::slotInstallClicked()
 */
void KInstaller::slotInstallClicked()
{
  QString pckdir, makedir;
  QString sourceFile;
  QDir dir;
  QFileInfo fi;
  QString configureOpts;
  bool doMove = true;
  

  // when a previous action is canceled but the cancel isn't
  // already processed we abort here
  if (vCanceled) return;

  // when a previous action is in progress we cancel here
  if (vRunning)
  {
    m_view->appendLog("user abort");
    m_view->toggleInstallButton(false,false);
    vRunning = false; vCanceled = true;
    return;
  }

  // here we start a new action
  processStart;
  
  m_view->clearLog();
  m_view->appendLog("\nchecking environment ...");
  
  // first we have a look if the source file is ok
  fi.setFile(m_view->sourceURL());
  if (!fi.exists())
  {
    m_view->appendLog("source file doesn't exist.");
    showError("Sorry, but the source file you selected doesn't seem to exist.\n"\
        "Please select the correct file.");
    processReady;
    return;
  }

  // a possible cancel request is processed here
  processCancelRequest;
  
  // here we set the destination directory
  switch (m_view->installType())
  {
    case KInstallerView::Temporary :
      m_view->appendLog(" using temporary directory for installation.");
      pckdir = m_view->tempDir();
      dir = QDir(pckdir);
      sourceFile = m_view->sourceURL();
      break;
    case KInstallerView::Persistent :
      m_view->appendLog(" using package directory for installation.");
      pckdir = m_view->pkgDir();
      dir = QDir(pckdir);
      sourceFile = dir.absFilePath(fi.fileName());
      
      // here we test if the source package already exists within the package directory
      fi.setFile(sourceFile);
      if (fi.exists())
      {
        if (KMessageBox::questionYesNo(this,
            "There exists already a tar package '" + fi.fileName()
                + "' at the package directory.\n"\
                "Do you want to overwrite it?") != KMessageBox::Yes)
        {
          doMove = false;
          processReady;
          return;
        }
      }

      break;
  }
  
  if (pckdir.isEmpty() || !dir.exists())
  {
    m_view->appendLog("installation directory doesn't exist.");
    showError("Sorry, but the destination directory you selected doesn't seem to exist.\n"\
        "Please select an existing installation directory.");
    processReady;
    return;
  }
  
  // a possible cancel request is processed here
  processCancelRequest;
  
  fi.setFile(dir.absPath());
  if (!fi.isWritable())
  {
    m_view->appendLog("installation directory isn't writable.");
    showError("Sorry, but the destination directory you selected isn't writable.\n"\
        "Please check for correct write access.");
    processReady;
    return;
  }

  // a possible cancel request is processed here
  processCancelRequest;
  
  // test for valid KDE base directory  
  if (!testKDEBaseDir(m_view->KDEBase()))
  {
    m_view->appendLog("KDE base directory isn't correct.");
    showError("Sorry, but the KDE base directory you selected isn't correct.\n"\
        "Please select the correct KDE base.");
    processReady;
    return;
  }
  
  // a possible cancel request is processed here
  processCancelRequest;
  
  // untar source file to destination folder
  m_view->appendLog("\nuntar source file ...");
  if (!untarSourceFile(m_view->sourceURL(),pckdir,makedir))
  {
    m_view->appendLog("Untaring the source file has failed.");
    showError("Sorry, but untaring the source file has failed.\n"\
        "Please consult the log entries for further informations.");
    processReady;
    return;
  }
  
  // a possible cancel request is processed here
  processCancelRequest;
  
  // configure
  m_view->appendLog("\nexecute 'configure' ...");
  m_view->appendLog("directory is " + makedir);

  if (m_view->useConfigureOptions())
    configureOpts = m_view->configureOptions();
  else
    configureOpts = "";
  
  if (!vProcess->exec(QString("cd %1; ./configure --prefix=%2 %3").arg(makedir).arg(m_view->KDEBase()).arg(configureOpts)))
  {
    m_view->appendLog("error execute configure");
    showError("An error occurred while executing configure.\n"\
        "Please consult the log entries for further informations.");
    processReady;
    return;
  }
  
  // a possible cancel request is processed here
  processCancelRequest;
  
  // make
  m_view->appendLog("\nexecute 'make' ...");
  
  if (!vProcess->exec(QString("cd %1; make").arg(makedir)))
  {
    m_view->appendLog("error execute make");
    showError("An error occurred while executing make.\n"\
        "Please consult the log entries for further informations.");
    processReady;
    return;
  }
  
  // a possible cancel request is processed here
  processCancelRequest;
  
  // make install
  m_view->appendLog("\nexecute 'make install' ...");
    
  if (!vProcess->exec("kdesu",QString("cd %1; make install").arg(makedir)))
  {
    m_view->appendLog("error while installing");
    showError("An error occurred while installing.\n"\
        "Please consult the log entries for further informations.");
    processReady;
    return;
  }
  
  // a possible cancel request is processed here
  processCancelRequest;
  
  // cleanup
  m_view->appendLog("\ncleaning up ...");
  
  switch (m_view->installType())
  {
    case KInstallerView::Temporary :
      // in temporary case we only need to delete the whole package directory
      if (!vProcess->exec("rm","-rf",QString("%1").arg(pckdir)))
      {
        m_view->appendLog("error while cleaning up");
        showError("An error occurred while incleaning up.\n"\
            "Please consult the log entries for further informations.");
        processReady;
        return;
      }
      break;
    case KInstallerView::Persistent :
      // at last delete the package directory
      if (!vProcess->exec("rm","-rf",QString("%1").arg(pckdir)))
      {
        m_view->appendLog("error while cleaning up");
        showError("An error occurred while incleaning up.\n"\
            "Please consult the log entries for further informations.");
        processReady;
        return;
      }
      
      if (doMove)
      {
        if (!vProcess->exec("mv","-f",
            QString("%1").arg(m_view->sourceURL()),
            QString("%1").arg(sourceFile)))
        {
          m_view->appendLog("error while cleaning up");
          showError("An error occurred while cleaning up.\n"\
              "Please consult the log entries for further informations.");
          processReady;
          return;
        }
      }
      
      break;
  }
  
  m_view->appendLog("\nready.");

  processReady;
}


/*!
    \fn KInstaller::testKDEBaseDir(const QString & path)
 */
bool KInstaller::testKDEBaseDir(const QString & path)
{
  QDir dir;
  QFileInfo fi;
  
  
  dir = QDir(path);
  fi.setFile(dir.absFilePath("bin/konqueror"));
  return(fi.exists());
}


/*!
    \fn KInstaller::showError(const QString & error)
 */
void KInstaller::showError(const QString & error)
{
  KMessageBox::error(this,error,i18n("error"));
}


/*!
    \fn KInstaller::showInfo(const QString & msg)
 */
void KInstaller::showInfo(const QString & msg)
{
  KMessageBox::information(this,msg,i18n("info"));
}


/*!
    \fn KInstaller::untarSourceFile(const QString & filename, QString & destDir, QString & makeDir)
 */
bool KInstaller::untarSourceFile(const QString & filename, QString & destDir, QString & makeDir)
{
  QDir dir(destDir);
  QString tmpDir = QString("kinstaller-%1").arg(QDateTime::currentDateTime().toString("yyyyMMddhhmmss"));
  QString flags;
  QStringList entries;
  int numEntries = 0;
  

  // first we create a temporary install sub directory
  dir = dir.absFilePath(tmpDir);
  destDir = dir.absPath();
  if (!dir.mkdir(dir.absPath()))
  {
    m_view->appendLog("cannot create directory");
    return(false);
  }

  // now we test for proper compress flag
  if (filename.endsWith("tar"))
    flags = "-x";
  else if (filename.endsWith("tar.gz"))
    flags = "-xz";
  else if (filename.endsWith("tgz"))
    flags = "-xz";
  else if (filename.endsWith("tar.bz2"))
    flags = "-xj";

  // yte the file is decompressed
  if (!vProcess->exec("tar",flags,"-C",dir.absPath(),"-f",filename))
  {
    m_view->appendLog("error while executing tar");
    return(false);
  }

  // now we are searching for the make directory
  entries = dir.entryList("*");
  for (QStringList::const_iterator it = entries.begin(); it != entries.end(); ++it)
  {
    if (*it != QString(".") && *it != QString(".."))
    {
//      qDebug(i18n("found sub directory " + *it).ascii());
      ++numEntries;
      if (numEntries == 1)
      {
        dir = dir.absFilePath(*it);
        makeDir = dir.absPath();
      }
      else
      {
        m_view->appendLog("source file contains more than one directory");
        return(false);
      }        
    }
  }
  
  return(true);
}


/*!
    \fn KInstaller::processCancelRequest(const QString & installDir)
 */
void KInstaller::processCancel(const QString & installDir)
{
  vProcess->requestAbort();
  m_view->toggleInstallButton(false,true);
  vRunning = false; vCanceled = false;
  if (!vProcess->exec("rm","-rf",QString("%1").arg(installDir)))
  {
    m_view->appendLog("error while cleaning up");
    showInfo("During installation the directory\n" + installDir + "\nhas been created."\
              " But now KInstall was't able to remove it.\n"\
              "Please remove it by hand.");
    return;
  }
  
  m_view->appendLog("operation successfully canceled.");
}


/*!
    \fn KInstaller::slotProcessStdout(const QString & str)
 */
void KInstaller::slotProcessStdout(const QString & str)
{
  m_view->appendLog(str);
}


/*!
    \fn KInstaller::slotProcessStderr(const QString & str)
 */
void KInstaller::slotProcessStderr(const QString & str)
{
  m_view->appendLog(str);
}


#include "kinstaller.moc"
