/***************************************************************************
 *   Copyright (C) 2004 by Ingo Stierand                                   *
 *   ing.stierand@web.de                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef _KINSTALLERVIEW_H_
#define _KINSTALLERVIEW_H_

#include <qwidget.h>
//#include <kinstalleriface.h>

#include "kinstallervw.h"


//class QPainter;
//class KURL;

/**
 * This is the main view class for KInstaller.  Most of the non-menu,
 * non-toolbar, and non-statusbar (e.g., non frame) GUI code should go
 * here.
 *
 * @short Main view
 */
class KInstallerView : public KInstallerVw
{
  Q_OBJECT
      
  public:
    typedef enum {
      Temporary = 1,
      Persistent
    } InstallType;
    
    /**
      * Default constructor
      */
    KInstallerView(QWidget *parent);

    /**
      * Destructor
      */
    virtual ~KInstallerView();

    /**
      * Returns the URL of the source package
      */
    QString sourceURL();
    void setSourceURL(const QString & url);

    /**
     * Returns the base directory of KDE
     */
    QString KDEBase();
    void setKDEBase(const QString & dir);

    /**
      * Returns the installation type. This is either
      * temporary or persistent.
     */
    InstallType installType();
    void setInstallType(InstallType type);
    
    /**
      * Returns the temp directory
      */
    QString tempDir();
    void setTempDir(const QString & dir);

    /**
      * Returns the package directory
      */
    QString pkgDir();
    void setPkgDir(const QString & dir);
    
    /**
      * Wether addtional configure options should be used.
      */
    bool useConfigureOptions();
    void setUseConfigureOptions(bool use);

    /**
      * Addtional configure options
      */
    QString configureOptions();
    void setConfigureOptions(const QString & url);

    void toggleInstallButton(bool running, bool enabled=true);

    void clearLog();
    void appendLog(const QString & text);
    
  signals:
    /**
     * Use this signal to change the content of the statusbar
     */
//    void signalChangeStatusbar(const QString& text);

    /**
     * Use this signal to change the content of the caption
     */
//    void signalChangeCaption(const QString& text);
    
    void signalInstallClicked();
    void signalSelectSourceClicked();
    void signalSelectTempDirClicked();
    void signalSelectPkgpDirClicked();
    void signalGuessKDEBaseClicked();

  private slots:
//    void slotOnURL(const QString& url);
//    void slotSetTitle(const QString& title);
    void slotInstallClicked();
    void slotSelectSourceClicked();
    void slotSelectTempDirClicked();
    void slotSelectPkgDirClicked();
    void slotGuessKDEBaseClicked();
    void slotRadioToggled(bool checked);
    void slotUseOptionsToggled(bool checked);

  private:
};

#endif // _KINSTALLERVIEW_H_
